// -*- C++ -*-
/*!
 * @file  PanTiltComp.cpp
 * @brief PanTiltComp
 * @date $Date$
 *
 * @author メディアドライブ
 * rtc-dev@mediadrive.jp
 *
 * BSD 3-clause "New" or "Revised" License
 *
 * $Id$
 */

#include "PanTiltComp.h"

#define TILT_MIN -110
#define TILT_MAX 110

#define LIGHT_PIN 0
#define SERVO_PIN 2
#define SLAVE_MODE

#define DEBUG_PRINT 1
#define DEBUG_MODE 1

// Module specification
// <rtc-template block="module_spec">
static const char* pantiltcomp_spec[] =
  {
    "implementation_id", "PanTiltComp",
    "type_name",         "PanTiltComp",
    "description",       "PanTiltComp",
    "version",           "1.0.0",
    "vendor",            "MEDIA DRIVE",
    "category",          "Motor",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

//
int set_pos (int id, int pos, int fd);

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
PanTiltComp::PanTiltComp(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_TargetCameraPlatformIn("TargetCameraPlatform", m_TargetCameraPlatform),
    m_NowCameraPlatformOut("NowCameraPlatform", m_NowCameraPlatform)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
PanTiltComp::~PanTiltComp()
{
}


/*!
 * 各変数の初期化を行います
 */
RTC::ReturnCode_t PanTiltComp::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("TargetCameraPlatform", m_TargetCameraPlatformIn);

  // Set OutPort buffer
  addOutPort("NowCameraPlatform", m_NowCameraPlatformOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

  // <rtc-template block="bind_config">
  // </rtc-template>

  if (wiringPiSetup () == -1)
      return RTC::RTC_ERROR;

  light_flag = 0;
  servo_flag = 0;
  fd = -1;

  return RTC::RTC_OK;
}

/*!
 * シリアル接続解除処理を行います
 */

RTC::ReturnCode_t PanTiltComp::onFinalize()
{
  if(servo_flag != 0)
{
	   //初期位置に復帰
	   digitalWrite (SERVO_PIN, 1);
	   set_pos (0, 7500, fd);
	   sleep(1);
	   set_pos (1, 4833, fd);
	   servo_flag = 0;
}
	  digitalWrite (SERVO_PIN, 0);
	  digitalWrite (LIGHT_PIN, 0);
  light_flag = 0;
	  pinMode (SERVO_PIN, INPUT);
	  pinMode (LIGHT_PIN, INPUT);

  if(0 < fd)
  {
	  serialClose (fd);
          fd = -1;
}
	  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PanTiltComp::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PanTiltComp::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * シリアル接続処理を行います
 */

RTC::ReturnCode_t PanTiltComp::onActivated(RTC::UniqueId ec_id)
{
  	digitalWrite (LIGHT_PIN, 0);
  	digitalWrite (SERVO_PIN, 0);
  	pinMode (SERVO_PIN, INPUT);
  	pinMode (LIGHT_PIN, INPUT);

  	pinMode (SERVO_PIN, OUTPUT);
  	pinMode (LIGHT_PIN, OUTPUT);
  	digitalWrite (LIGHT_PIN, 0);
        light_flag = 0;
  	//digitalWrite (SERVO_PIN, 1);


if(servo_flag == 0)
{
	/* シリアルポートオープン */
	fd = serialOpen ("/dev/ttyAMA0", 115200);
	//fd = serialOpen ("/dev/ttyUSB0", 115200);
	if (fd < 0)
	{
		//std::cout << "can not open serialport." << std::endl;
		printf ("can not open serialport.\n");
		return RTC::RTC_ERROR;
	}

	struct termios options;
	//通信設定(データ長8ビット,偶数パリティ)
	tcgetattr (fd, &options);	// Read current options
	options.c_cflag &= ~CSIZE;	// Mask out size
	options.c_cflag |= CS8;	// Or in 8-bits
	options.c_cflag |= PARENB;	// Enable Parity - even by default
	tcsetattr (fd, TCSANOW, &options);	// Set new options

        servo_flag = 1;
}

    //初期位置に復帰
    digitalWrite (SERVO_PIN, 1);
    set_pos (0, 7500, fd);
	sleep(1);
    set_pos (1, 4833, fd);
	sleep(1);

  return RTC::RTC_OK;
}

/*!
 * シリアル接続解除処理を行います
 */

RTC::ReturnCode_t PanTiltComp::onDeactivated(RTC::UniqueId ec_id)
{
  if(servo_flag != 0)
{
   //初期位置に復帰
   digitalWrite (SERVO_PIN, 1);
   set_pos (0, 7500, fd);
   sleep(1);
   set_pos (1, 4833, fd);
   sleep(1);
   servo_flag = 0;

  if(0 < fd)
  {
    serialClose (fd);
    fd = -1;
  }

  digitalWrite (SERVO_PIN, 0);
  pinMode (SERVO_PIN, INPUT);
}
  digitalWrite (LIGHT_PIN, 0);
  light_flag = 0;
  pinMode (LIGHT_PIN, INPUT);
  return RTC::RTC_OK;
}

/*!
 * InPortから受け取った命令をカメラ雲台に送りパンチルト操作、ライト
 * 点灯を行います
 */

RTC::ReturnCode_t PanTiltComp::onExecute(RTC::UniqueId ec_id)
{
        int num;
        int command[2];
	if(m_TargetCameraPlatformIn.isNew())
	{
                // data[0]                data[1]
		// 0: Pan  Servo          0=OFF 0<>ON
		// 1: Tilt Servo          0=OFF 0<>ON
		// 2: Pan  Value          value
		// 3: Tilt Value          value
		// 4: Light Value         0=OFF 0<>ON
		//99: Pan,Tilt Reset;

		m_TargetCameraPlatformIn.read();

#ifdef DEBUG_PRINT
	    std::cout << m_TargetCameraPlatform.data.length() << " datas received." << std::endl;
	    for(int i = 0;i < m_TargetCameraPlatform.data.length();i++)
	    {
	      std::cout << i << ": Data is " << m_TargetCameraPlatform.data[i] << std::endl;
	    }
#endif
	    for(int i = 0;i < 2;i++)
	    {
              command[i] = m_TargetCameraPlatform.data[i];
	    }
           int cnum = command[0];

#ifdef DEBUG_PRINT
	   std::cout << "switch()" << std::endl;
	      std::cout << command[0] << std::endl;
	      std::cout << command[1] << std::endl;
#endif
	   switch( cnum  )
	   {
	   	case 0:
	    	//case 1:
			    	if(command[1] == 1)
			    	{
			    		if(servo_flag == 0)
			    		{
			    			// SERVO ON
#ifdef DEBUG_PRINT
	      					std::cout << "SERVO ON" << std::endl;
#endif
  pinMode (SERVO_PIN, OUTPUT);
  digitalWrite (SERVO_PIN, 1);
  if(0 < fd)
  {
    serialClose (fd);
    fd = -1;
  }
	/* シリアルポートオープン */
	fd = serialOpen ("/dev/ttyAMA0", 115200);
	//fd = serialOpen ("/dev/ttyUSB0", 115200);
	if (fd < 0)
	{
		//std::cout << "can not open serialport." << std::endl;
		printf ("can not open serialport.\n");
		return RTC::RTC_ERROR;
	}

	struct termios options;
	//通信設定(データ長8ビット,偶数パリティ)
	tcgetattr (fd, &options);	// Read current options
	options.c_cflag &= ~CSIZE;	// Mask out size
	options.c_cflag |= CS8;	// Or in 8-bits
	options.c_cflag |= PARENB;	// Enable Parity - even by default
	tcsetattr (fd, TCSANOW, &options);	// Set new options

   //初期位置に復帰
   digitalWrite (SERVO_PIN, 1);
   set_pos (0, 7500, fd);
   sleep(1);
   set_pos (1, 4833, fd);
			    			servo_flag = 1;
			    		}
			    	}
			    	else
			    	{
			    		// SERVO FREE and OFF
			    		if(servo_flag == 1)
			    		{
#ifdef DEBUG_PRINT
	      					std::cout << "SERVO OFF" << std::endl;
#endif
   //初期位置に復帰
   digitalWrite (SERVO_PIN, 1);
   set_pos (0, 7500, fd);
   sleep(1);
   set_pos (1, 4833, fd);
   sleep(1);
  if(0 < fd)
  {
    serialClose (fd);
    fd = -1;
  }
  digitalWrite (SERVO_PIN, 0);
  pinMode (SERVO_PIN, INPUT);
			    			servo_flag = 0;
			    		}
			    	}
			    	break;
	    		case 2:
	    			if(servo_flag == 1)
	    			{
	    				num = command[1];
	    				  position = (int) (7500 + (num * 4000) / 135);
	    				  if (position < 3500)
	    				    position = 3500;
	    				  if (11500 < position)
	    				    position = 11500;

#ifdef DEBUG_PRINT
	      					std::cout << "PAN " << num << std::endl;
#endif
	    				  /* 送信処理 */
	    				  //serialFlush(fd);
	    				  set_pos (0, position, fd);
	    			}
	    			break;
	    		case 3:
	    			if(servo_flag == 1)
	    			{
	    				num = command[1];
	    				  if (num < TILT_MIN)
	    				    num = TILT_MIN;
	    				  if (TILT_MAX < num)
	    				    num = TILT_MAX;
	    				  position = (int) (7500 + ((num - 90) * 4000) / 135);

	    				  if (position < 3500)
	    				    position = 3500;
	    				  if (11500 < position)
	    				    position = 11500;
#ifdef DEBUG_PRINT
	      				std::cout << "TILT " << num << std::endl;
#endif
	    				  /* 送信処理 */
	    				  set_pos (1, position, fd);
	    			}
	    			break;
	    		case 4:
			    	if(command[1] == 1)
			    	{
#ifdef DEBUG_PRINT
	      				std::cout << "LIGHT ON" << std::endl;
#endif
		    			// LIGHT ON
		    			digitalWrite(LIGHT_PIN, 1);
		    			light_flag = 1;
			    	}
			    	else
			    	{
#ifdef DEBUG_PRINT
	      				std::cout << "LIGHT OFF" << std::endl;
#endif
			    		// LIGHT OFF
		    			light_flag = 0;
		    			digitalWrite(LIGHT_PIN, 0);
			    	}
			    	break;
			case 99:
		    		if(servo_flag == 1)
				{
					//初期位置に復帰
					//digitalWrite (SERVO_PIN, 1);
					set_pos (0, 7500, fd);
					sleep(1);
					set_pos (1, 4833, fd);
					sleep(1);
				}
				break;
	    		default:
#ifdef DEBUG_PRINT
	      					std::cout << "Unknown command" << std::endl;
#endif
	    			break;
	    	}
	   std::cout << "end switch()" << std::endl;
	    }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PanTiltComp::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PanTiltComp::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PanTiltComp::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PanTiltComp::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PanTiltComp::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


int
set_pos (int id, int pos, int fd)
{
	//unsigned char tx[3];
	//unsigned char rx[3];
	char tx[3];
	int i;
	tx[0] = 0x80 | id;		// CMD
	tx[1] = /* (unsigned char) */ (pos >> 7 & 0x7F);	// POS_H
	tx[2] = /* (unsigned char) */ (pos & 0x7F);	// POS_L
        int retnum;

	//バッファをクリアする

#ifdef DEBUG_MODE
	if (id == 0)
		printf ("Pan ");
	else
		printf ("Tilt ");
	printf ("position : %d\n", pos);
	printf ("send : \n");
	for (i = 0; i < 3; i++)
	{
		printf ("%02X ", (int) tx[i]);
	}
	printf ("\n");
#endif

	// tx配列(ICSコマンド)をシリアルポートから出力する
  	for (i = 0; i < 3; i++)
    	{
      	serialPutchar (fd, tx[i]);
    	}
#ifndef SLAVE_MODE
  	printf ("receive : \n");
  	for (i = 0; i < 3; i++)
    	{
      	printf ("%02X ", serialGetchar (fd));
    	}
  	printf ("\n");
#endif
	return 0;
}



extern "C"
{

  void PanTiltCompInit(RTC::Manager* manager)
  {
    coil::Properties profile(pantiltcomp_spec);
    manager->registerFactory(profile,
                             RTC::Create<PanTiltComp>,
                             RTC::Delete<PanTiltComp>);
  }

};


